@@ -58,16 +58,19 @@ def create_user_gti(time, in_rate, in_drate, lognormal=True, gti_name='user_gtis
 	if lognormal:
 		rate = np.log10(in_rate)
 		drate = in_drate/in_rate
+		rate_label = '$\\log_{10}(\\mathrm{rate})$'
 	else:
 		rate = in_rate
 		drate = in_drate
-        
+		rate_label = 'rate'
+    
+
 	limit_down, limit_up = utils.rate_filter(time, rate, drate, 
                                        level_down=level_down, 
                                        level_up = level_up, 
                                        quantile=quantile, 
                                        make_plot=make_plot, 
-                                       far=far)
+                                       far=far, rate_label=rate_label)
     
 	new_gti = create_gti_from_condition(time, (rate > limit_down) & (rate < limit_up) )
 
@@ -129,6 +132,218 @@ def get_data(obsid):
 	cmd="wget -q -nH --no-check-certificate --cut-dirs=6 -r -l0 -c -N -np -R 'index*' -erobots=off --retr-symlinks https://heasarc.gsfc.nasa.gov/FTP/nustar/data/obs/"+obsid[1:3]+"/"+obsid[0]+"/"+obsid+"/"
 	return run(cmd)
 
+def extract_counts(img, nominal_y, nominal_x, pix_region):
+	
+	from photutils import CircularAperture
+	from photutils import aperture_photometry
+
+	aperture = CircularAperture((nominal_x, nominal_y), r=pix_region)
+	# aperture = CircularAnnulus((nominal_x, nominal_y), r_in=pix_region, r_out=pix_region+1)
+
+	phot_table = aperture_photometry(img, aperture)
+	#print(phot_table)
+	return phot_table['aperture_sum'].data[0]  # , aperture.area
+
+def prepare_region(unit, default_psf_arcsec=120., criterion='95', critical_surface_brightness=1, use_max_coord = False, n_bins=10, 
+						min_psf = 0.1, max_psf = 2, forced_r_pix = 0, back_offset = 3, make_plots=True):
+	from astropy.io import fits as pf
+	from astropy.wcs import WCS
+	import numpy as np
+
+	import matplotlib.pyplot as plt
+
+	from photutils import CircularAperture
+	from photutils import CircularAnnulus
+	from photutils import aperture_photometry
+
+	from astropy.visualization.mpl_normalize import ImageNormalize
+	from astropy.visualization import LogStretch
+
+	from IPython.display import Image
+	from IPython.display import display
+
+	default_psf_deg = default_psf_arcsec / 3600.
+
+	filename='ima%s.fits' % unit
+	f_header = pf.open(filename)
+	hdu = f_header[0]
+	my_wcs = WCS(hdu.header)
+	image = hdu.data
+
+	ra = hdu.header['RA_OBJ']
+	dec = hdu.header['DEC_OBJ']
+
+
+	xscale = np.abs(hdu.header['CDELT1P'])
+	yscale = np.abs(hdu.header['CDELT2P'])
+
+	xscale_wcs = np.abs(hdu.header['CDELT1'])
+	yscale_wcs = np.abs(hdu.header['CDELT2'])
+
+	ra_pnt = hdu.header['RA_PNT']
+	dec_pnt = hdu.header['DEC_PNT']
+
+	f_header.close()
+
+	ind_max = np.unravel_index(np.argmax(image, axis=None), image.shape)
+	print("Image maximum is at ", my_wcs.wcs_pix2world(ind_max[1], ind_max[0], 0))
+
+	if use_max_coord:
+		nominal_x = ind_max[1]
+		nominal_y = ind_max[0]
+	else:
+		nominal_x, nominal_y = my_wcs.all_world2pix(ra, dec, 0)
+		
+	centre_x, centre_y = my_wcs.all_world2pix(ra_pnt, dec_pnt, 0)
+
+	scale_matrix = my_wcs.pixel_scale_matrix
+	pix_region = default_psf_deg / scale_matrix[1, 1]
+	delta_r = pix_region / n_bins
+
+	if make_plots:
+		fig1 = plt.figure()
+		ax = plt.subplot(projection=my_wcs)
+		minv, maxv = np.percentile(image, [5, 95])
+		img_norm = ImageNormalize(image, vmin=minv, vmax=maxv, stretch=LogStretch(10))
+		plt.imshow(image, norm=img_norm)  # , vmin=-2.e-5, vmax=2.e-4, origin='lower')
+		plt.colorbar()
+		ax.grid(color='white', ls='solid')
+		ax.set_xlabel('RA')
+		ax.set_ylabel('Dec')
+		
+		# PLOTS THE default SRC REGION WITH 90% CONFINEMENT
+		ax.scatter(nominal_x, nominal_y, s=40, marker='x', color='red')
+
+		circle = plt.Circle([nominal_x, nominal_y], pix_region, color='red', fill=False, linestyle='--')
+		ax.add_artist(circle)
+	
+	# Computes the PSF
+	r_psf = np.linspace(1, 2 * int(np.floor(pix_region)), 2 * int(np.floor(pix_region)))
+	# psf = [(x, extract_counts(image, nominal_y, nominal_x, x)) for x in r_psf]
+	# apertures = [CircularAperture((nominal_x, nominal_y), r=x) for x in r_psf]
+	apertures = [CircularAnnulus((nominal_x, nominal_y), r_in=x, r_out=x + delta_r) for x in r_psf]
+	back_apertures = [CircularAnnulus((nominal_x, nominal_y), r_in=x + delta_r, r_out=x + 2 * delta_r) for x in r_psf]
+
+	phot_table = aperture_photometry(image, apertures)
+	back_phot_table = aperture_photometry(image, back_apertures)
+
+	aperture_areas = np.array([aa.area for aa in apertures])
+	back_aperture_areas = np.array([aa.area for aa in back_apertures])
+
+	my_keys = [x for x in phot_table.keys() if 'aperture_sum' in x]
+	my_back_keys = [x for x in back_phot_table.keys() if 'aperture_sum' in x]
+
+	c_psf = np.array([phot_table[kk][0] for kk in my_keys])
+	c_psf /= aperture_areas
+
+	back_c_psf = np.array([phot_table[kk][0] for kk in my_back_keys])
+	back_c_psf /= back_aperture_areas
+
+	# psf_back = [(x, extract_counts(image, back_y, back_x, x)) for x in r_psf]
+	psf_back = [(x, y) for x,y in zip(r_psf, back_c_psf)]  # *x**2
+	# gets and plots the maximum S/N
+	# r_psf = [x[0] for x in psf]
+	# print(r_psf)
+	# c_psf = [x[1] for x in psf]
+	r_back = np.array([x[0] for x in psf_back])
+	c_back = np.array([x[1] for x in psf_back])
+
+	c_net = np.array(c_psf) - np.array(c_back)
+	# scan change of slope
+	last_ind = 2
+	for i in range(2, len(c_net)):
+		last_ind = i
+		if c_net[i] >= np.mean(c_net[i - 2:i - 1]):
+			print('found change of slope in PSF at index %d' % i)
+			break
+
+	real_psf = np.array([extract_counts(image, nominal_y, nominal_x, x) for x in r_psf])
+	real_psf_net = real_psf - c_back * r_back ** 2
+
+	norm_cum_sum = real_psf_net / real_psf_net.max()
+
+	gradient = np.gradient(real_psf_net)
+	# back_gradient = np.gradient(c_back)
+	gradient /= gradient.max()
+
+	# gradient2 = np.gradient(gradient)
+	# gradient2 /= np.max(np.abs(gradient))
+	# s_n = c_net * aperture_areas / np.sqrt(c_psf * aperture_areas)
+	s_n = real_psf_net / np.sqrt(real_psf)
+
+	if make_plots:
+		fig2, ax_2 = plt.subplots(1, 2, sharex=True)
+		ax_2[0].plot(r_psf, c_psf, color='blue', label="anulus brillance")
+		ax_2[0].plot(r_back, c_back, color='red', label="background brillance")
+		ax_2[0].plot(r_back, c_net, color='green', label="anulus net brillance")
+
+		# ax_2[0].plot(r_psf, real_psf, color='blue', label="Cum PSF")
+		# ax_2[0].plot(r_back, c_back*r_back**2, color='red', label="background")
+		# ax_2[0].plot(r_back, real_psf_net, color='green', label="Net PSF")
+
+		#ax_2[0].set_title(self.target + ' ' + self.obs_id)
+		ax_2[0].legend()
+
+		ax_2[1].plot(r_psf, norm_cum_sum, color='black', label="Cum PSF")
+
+		ax_2[1].plot(r_psf, gradient, color='blue', label='PSF')
+		# ax_2[1].plot(r_psf, back_gradient, color='red', label='back gradient')
+		# ax_2[1].plot(r_psf, gradient2, color='green', label='gradient2')
+
+		ax_2[1].plot(r_psf, s_n / s_n.max(), color='cyan', label='S/N')
+
+		ax_2[1].grid(b=True)
+		ax_2[1].legend()
+
+	ind_95 = int(np.argmin(np.abs(c_net[0:last_ind] - critical_surface_brightness)) + delta_r / 2)
+	if ind_95 >= len(r_psf):
+		ind_95 = len(r_psf) - 1
+
+	if criterion == '95':
+		ind_95 = np.argmin(np.abs(norm_cum_sum - 0.95))
+
+	if make_plots:
+		ax_2[1].scatter(r_psf[ind_95], norm_cum_sum[ind_95], marker='x', color='red')
+
+	r_max = r_psf[ind_95]
+	c_max = c_net[ind_95]
+
+	print("Optimal inclusion is r=%.1f +/- %.1f with %.1f cts/pix " % (r_max, delta_r / 2, c_max))
+	print("Cumulative PSF is %.2f" % norm_cum_sum[ind_95])
+
+	if r_max < min_psf*pix_region:
+		print('r_max is lower than %.1f' % min_psf)
+		r_max = min_psf
+
+	if r_max > max_psf * pix_region:
+		print('r_max is larger than %.1f' % max_psf)
+		r_max = max_psf
+
+	if forced_r_pix > 0:
+		r_max = forced_r_pix
+		print("Input forced radius for extraction region at %.1f" % r_max)
+
+	if make_plots:
+		# plt.scatter(r_max,c_max,s=60,marker='x', color='red')
+
+		ax_2[0].axvline(r_max, 0, 1, color='black', linestyle='--', linewidth=2)
+		ax_2[1].axvline(r_max, 0, 1, color='black', linestyle='--', linewidth=2)
+
+	distance = np.sqrt( (centre_x - nominal_x)**2 + (centre_y - nominal_y)**2 )
+	back_offset *= r_max / distance 
+	back_x = centre_x + back_offset * (centre_x - nominal_x)
+	back_y = centre_y + back_offset * (centre_y - nominal_y)
+	if make_plots:
+		circle = plt.Circle([back_x, back_y], pix_region, color='white', fill=False)
+		ax.add_artist(circle)
+		circle = plt.Circle([nominal_x, nominal_y], r_max, color='red', fill=False)
+		ax.add_artist(circle)
+	
+	r_max_arcsec = r_max * scale_matrix[1, 1] * 3600.
+	ra_back, dec_back = my_wcs.all_pix2world(back_x,back_y, 0)
+
+	return ra, dec, r_max_arcsec, ra_back, dec_back
+
 def write_region(fname, ra, dec, src_flag, ra_back=-1, dec_back=-1, radius=120):
 	ff = open(fname, 'w')
 	ff.write("global color=green dashlist=8 3 width=1 font=\"helvetica 10 normal roman\" select=1 highlite=1 dash=0 fixed=0 edit=1 move=1 delete=1 include=1 source=1\nfk5\n")
@@ -274,20 +489,55 @@ def wrap_process(obsid, ra_src, dec_src, outdir_base, pipeline_flag=False, regio
 		outdir_pipeline = outdir_base + "_pipeline"
 		prepare_outdir(outdir)
 
+		logger.info('Extracting images')
+		xsel_cmd = '''dummy
+		read eve
+		./ 
+		nu%s%s01_cl.evt 
+		yes
+		filter pha_cut %d %d
+		extra ima
+		save ima ima%s.fits
+
+
+
+
+		quit
+
+
+
+
+
+		'''
+
+		chan_min = get_channel(e_min)
+		chan_max = get_channel(e_max)
+		
+		os.chdir(outdir_pipeline)
+		for unit in ['A', 'B']:
+			with open('xsel_%s.txt' % unit, 'w') as ff:
+				ff.write(xsel_cmd % (obsid, unit, chan_min, chan_max, unit))
+			cmd = 'xselect < ' + 'xsel_%s.txt' % (unit)
+			run(cmd)
+		os.chdir('..')
+
+		raA, decA, r_max_arcsecA, ra_backA, dec_backA = prepare_region('A')
+		raB, decB, r_max_arcsecB, ra_backB, dec_backB = prepare_region('B')
+
 		logger.info("We store regions in the spectral extraction folder '%s'" % outdir)
 
 		src_regionA = outdir + "/sourceA.reg"
 		if (not os.path.isfile(src_regionA)):
-			write_region(src_regionA, ra_src, dec_src, True)
+			write_region(src_regionA, ra_src, dec_src, True, radius=r_max_arcsecA)
 		src_regionB = outdir + "/sourceB.reg"
 		if (not os.path.isfile(src_regionB)):
-			write_region(src_regionB, ra_src, dec_src, True)
+			write_region(src_regionB, ra_src, dec_src, True, radius=r_max_arcsecB)
 		bkg_regionA = outdir + "/backgroundA.reg"
 		if (not os.path.isfile(bkg_regionA)):
-			write_region(bkg_regionA, ra_src, dec_src, False, ra_back=ra_back, dec_back=dec_back, radius=radius)
+			write_region(bkg_regionA, ra_src, dec_src, False, ra_back=ra_backA, dec_back=dec_backA, radius=r_max_arcsecA)
 		bkg_regionB = outdir + "/backgroundB.reg"
 		if (not os.path.isfile(bkg_regionB)):
-			write_region(bkg_regionB, ra_src, dec_src, False, ra_back=ra_back, dec_back=dec_back, radius=radius)
+			write_region(bkg_regionB, ra_src, dec_src, False, ra_back=ra_backB, dec_back=dec_backB, radius=r_max_arcsecB)
 
 
 		if no_ds9_flag==False:
@@ -307,32 +557,6 @@ def wrap_process(obsid, ra_src, dec_src, outdir_base, pipeline_flag=False, regio
 		else:
 			# JS9 does not load event files, I need to create an image first
 
-			xsel_cmd = '''dummy
-			read eve
-			./ 
-			nu%s%s01_cl.evt 
-			yes 
-			extra ima
-			save ima ima%s.fits
-
-
-
-
-			quit
-
-
-
-
-
-			'''
-			os.chdir(outdir_pipeline)
-			for unit in ['A', 'B']:
-				with open('xsel_%s.txt' % unit, 'w') as ff:
-					ff.write(xsel_cmd % (obsid, unit, unit))
-				cmd = 'xselect < ' + 'xsel_%s.txt' % (unit)
-				run(cmd)
-			os.chdir('..')
-
 			logger.warning("Please check the regions for FPMA")
 
 			if J_A is None or J_B is None:
