@@ -92,10 +92,11 @@ def make_basic_fit():
     ff.close()
 
     #logger.debug(xspec_commands)
-    run('xspec - files-auto.xcm')
+    status = run('xspec - files-auto.xcm')
     from IPython.display import Image
     from IPython.display import display
     _ = display(Image(filename='basic-plot.gif_2', format="gif"))
+    return status
 
 def plot_periodogram():
     with open('ef_pipe_periodogram_f.qdp') as ff:
@@ -801,6 +802,7 @@ def fit_pulsed_frac(en, den, pf, dpf, stem=None, degree_pol=4, n_gauss=0, center
         ax1.plot(en, out.best_fit, '-', label='best fit', color=col[0])
         ax1.plot(en, comps['poly_'], '--', label='Polynomial', color=col[1])
         ax1.legend(loc='upper left')
+        ax1.set_xscale('log')
         ax2.set_xlabel('E (keV)')
         ax1.set_ylabel('PF')
         ax2.set_ylabel('Residuals')
@@ -942,7 +944,7 @@ def plot_matrix_as_image(ee, pp, kind='E', normalize=True,outfile=None, cmap=mat
             pp1[i, :] = (x - m) / s
 
     phi = np.linspace(0, 1, pp.shape[1])
-    plt.figure()
+    plt.figure(figsize=(8.8, 7))
     # if sliders:
     #     plt.subplots_adjust(top=0.82)
     if min_level is None:
@@ -1098,42 +1100,41 @@ def get_fourier_coeff_error( counts, counts_err, n_simul=100, use_poisson=False)
 
     return np.std(sim_phi0), np.std(sim_phi0_2,), np.std(sim_A), np.std(sim_A2)
 
-def rate_filter(time, rate, drate, level_down=100, level_up = 0, quantile=0.8, make_plot=True, 
-                far=1e-4):
-    help = '''
-sas_cr_filter(self, typical_level=0.5, quantile=0.7,  make_plot=True, far=1e-3,
+def rate_filter(time, rate, drate, level_down=0, level_up = 1e10, quantile=0.8, make_plot=True, 
+                far=1e-4, rate_label='rate'):
+    '''
+rate_filter(self, typical_level=0.5, quantile=0.7,  make_plot=True, far=1e-3,
                     tmin=0,tmax=1e12):
-This function prepares cleaned events by apllying standard filters for PN and MOS 
-(technically ev_fileter_expressions , are inherited from the general class and specialized in the derived classe) 
-cleans for CR flares if apply_rate_filter=True
-It is possible to select a time filter by setting tmin and tmax. This is useful if one want to eliminate a noise part
-of observation.
+This function prepares cleaned events by apllying a standard filter eliminating bins with FAR < far
+This is useful if one want to eliminate a peculiar part of an observation.
 The filter is controlled in the following way:
-- a light curve is extracted in time bins of 100 s above 10 keV
-- a Gaussian is fitted on LC data within the lower quantile (default=0.8) of the light curve
-- a limit is set on FAR, but it can be overrided by using the typical_level parameter, which is the lowest possible limit.
+- a Gaussian is fitted on LC data within the innner quantile (default=0.8) of the light curve
+- a limit is set on FAR, but it can be overrided by using the level_up and level_down parameters, which are the lowest/highest possible limit.
         '''
         
-    print(help)
+    print(rate_filter.__doc__)
 
     if make_plot:
         f, axes = plt.subplots(1, 2)
         axes[0].errorbar(time, rate, yerr=drate, linestyle='none')
+        axes[0].set_ylabel(rate_label)
         #axes[0].set_title(self.target + ' ' + self.obs_id)
         axes[0].set_xlabel('Time [s]')
         n_hist, edges, patches = axes[1].hist(rate, bins=300, density=True, facecolor='green',
                                               alpha=0.75)
+        axes[1].set_xlabel(rate_label)
+
     # x=(edges[0:-2]+edges[1:])/2
     from scipy.stats import norm
-    ind = rate <= np.quantile(rate, quantile)
+    ind = (rate <= np.quantile(rate, quantile)) & (rate >= np.quantile(rate, 1.-quantile))
     (mu, sigma) = norm.fit(rate[ind])
 
     y = norm.pdf(edges, mu, sigma)
 
     fap = np.min([1. / len(rate), far])
     tmp = norm.isf(fap, loc=mu, scale=sigma)
-    limit_up = np.max([level_up, tmp ])
-    limit_down = np.min([level_down, 2*mu-tmp])
+    limit_up = np.min([level_up, tmp ])
+    limit_down = np.max([level_down, 2*mu-tmp])
 
     if make_plot:
         _ = axes[1].plot(edges, y, 'r--', linewidth=2)
